/*
 *  Copyright (C) 2007 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>

#include <pspctrl.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspiofilemgr.h>

#include "psp_global.h"
#include "psp_sdl.h"
#include "psp_menu.h"
#include "psp_fmgr.h"
#include "psp_battery.h"
#include "psp_menu_help.h"
#include "psp_kbd.h"
#include "psp_chess.h"

# define MENU_GAME_MODE    0
# define MENU_PLAYER_COLOR 1
# define MENU_SIDE_COLOR   2
# define MENU_SHOW_COORD   3
# if 0
# define MENU_TIME_LIMIT   4
# define MENU_MOVE_LIMIT   5
# endif
# define MENU_DEPTH        4
# define MENU_BOOK         5
# define MENU_CLOCK        6
# define MENU_SCREENSHOT   7

# define MENU_HELP         8

# define MENU_NEW_GAME     9
# define MENU_RESIGN_GAME 10
# define MENU_BACK        11
# define MENU_SAVE_CFG    12
# define MENU_EXIT        13

# define MAX_MENU_ITEM (MENU_EXIT + 1)

  static menu_item_t menu_list[] =
  { 
    { "Game mode        : " },
    { "Player color     : " },
    { "Bottom color     : " },
    { "Show coordinates : " },
# if 0
    { "Time limit       : " },
    { "Move limit       : " },
# endif
    { "Search depth     : " },
    { "Book mode        : " },
    { "Clock frequency  : " },
    { "Save Screenshot  : " },

    { "Help" },

    { "New game" },
    { "Resign game" },
    { "Back to game" },
    { "Save config" },
    { "Exit" },
  };

  static int cur_menu_id = MENU_BACK;

  static int psp_menu_dirty = 1;

  static int chess_player_color = white;
  static int chess_side_color   = white;
  static int chess_game_mode    = GCHESS_HUMAN_VS_COMPUTER;
  static int chess_show_coord   = 0;
  static int chess_search_depth = 0;
  static int chess_book         = BOOKOFF;
# if 0 //LUDO: TO_BE_DONE !
  static int chess_move_limit   = 0;
  static int chess_time_limit   = 0;
# endif
  static int psp_cpu_clock      = 222;

void
string_fill_with_space(char *buffer, int size)
{
  int length = strlen(buffer);
  int index;

  for (index = length; index < size; index++) {
    buffer[index] = ' ';
  }
  buffer[size] = 0;
}

void
psp_display_screen_menu_battery(void)
{
  char buffer[64];

  int Length;
  int color;

  snprintf(buffer, 50, " [%s] ", psp_get_battery_string());
  Length = strlen(buffer);

  if (psp_is_low_battery()) color = PSP_MENU_RED_COLOR;
  else                      color = PSP_MENU_GREEN_COLOR;

  psp_sdl_back2_print(240 - ((8*Length) / 2), 6, buffer, color);
}

static void 
psp_display_screen_menu(void)
{
  char buffer[32];
  int menu_id = 0;
  int color   = 0;
  int x       = 0;
  int y       = 0;
  int y_step  = 0;

  //if (psp_menu_dirty) 
  {
    psp_sdl_blit_background();
    psp_menu_dirty = 0;

    psp_sdl_draw_rectangle(10,10,459,249,PSP_MENU_BORDER_COLOR,0);
    psp_sdl_draw_rectangle(11,11,457,247,PSP_MENU_BORDER_COLOR,0);

    psp_sdl_back2_print( 30, 6, " Start+L+R: EXIT ", PSP_MENU_WARNING_COLOR);

    psp_display_screen_menu_battery();

    psp_sdl_back2_print( 360, 6, " R: New Game ", PSP_MENU_NOTE_COLOR);

    psp_sdl_back2_print(30, 254, " []: Cancel  O/X: Valid  Select: Back ", 
                       PSP_MENU_BORDER_COLOR);

    psp_sdl_back2_print(370, 254, " By Zx-81 ",
                       PSP_MENU_AUTHOR_COLOR);
  }
  
  x      = 20;
  y      = 25;
  y_step = 10;
  
  for (menu_id = 0; menu_id < MAX_MENU_ITEM; menu_id++) {
    color = PSP_MENU_TEXT_COLOR;
    if (cur_menu_id == menu_id) color = PSP_MENU_SEL_COLOR;
    else 
    if (menu_id == MENU_EXIT) color = PSP_MENU_RED_COLOR;
    else
    if (menu_id == MENU_HELP) color = PSP_MENU_GREEN_COLOR;

    psp_sdl_back2_print(x, y, menu_list[menu_id].title, color);

    if (menu_id == MENU_PLAYER_COLOR) {
      if (chess_player_color == black) strcpy(buffer, "black");
      else                             strcpy(buffer, "white");
      string_fill_with_space(buffer, 5);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_SIDE_COLOR) {
      if (chess_side_color == black) strcpy(buffer, "black");
      else                           strcpy(buffer, "white");
      string_fill_with_space(buffer, 5);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_DEPTH) {
      if (chess_search_depth) {
        sprintf(buffer, "%d", chess_search_depth);
      } else {
        strcpy(buffer, "auto");
      }
      string_fill_with_space(buffer, 5);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
# if 0 //LUDO: TO_BE_DONE !
    if (menu_id == MENU_TIME_LIMIT) {
      if (chess_time_limit) {
        sprintf(buffer, "%d min", chess_time_limit);
      } else {
        strcpy(buffer, "none");
      }
      string_fill_with_space(buffer, 10);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_MOVE_LIMIT) {
      if (chess_move_limit) {
        sprintf(buffer, "%d", chess_move_limit);
      } else {
        strcpy(buffer, "none");
      }
      string_fill_with_space(buffer, 10);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
# endif
    if (menu_id == MENU_BOOK) {

      if (chess_book == BOOKOFF) strcpy(buffer, "off");
      else
      if (chess_book == BOOKRAND) strcpy(buffer, "random");
      else
      if (chess_book == BOOKBEST) strcpy(buffer, "best");
      else
      if (chess_book == BOOKWORST) strcpy(buffer, "worst");
      else
      /* if (chess_book == BOOKPREFER) */ strcpy(buffer, "prefer");

      string_fill_with_space(buffer, 15);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_GAME_MODE) {
      if (chess_game_mode == GCHESS_HUMAN_VS_COMPUTER) strcpy(buffer, "Human vs PSP");
      else
      if (chess_game_mode == GCHESS_HUMAN_VS_HUMAN) strcpy(buffer, "Human vs Human");
      else                                    strcpy(buffer, "PSP vs PSP");
      string_fill_with_space(buffer, 15);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_SHOW_COORD) {
      if (chess_show_coord) strcpy(buffer, "yes");
      else                  strcpy(buffer, "no");
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_CLOCK) {
      sprintf(buffer,"%d", psp_cpu_clock);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(170, y, buffer, color);
    } else
    if (menu_id == MENU_SCREENSHOT) {
      sprintf(buffer,"%d", GCHESS.psp_screenshot_id);
      string_fill_with_space(buffer, 4);
      psp_sdl_back2_print(170, y, buffer, color);
      y += y_step;
    } else
    if (menu_id == MENU_SAVE_CFG) {
      y += y_step;
    } else
    if (menu_id == MENU_HELP) {
      y += y_step;
    } else
    if (menu_id == MENU_EXIT) {
      y += y_step;
    }


    y += y_step;
  }
}

void
psp_main_menu_new_game(void)
{
  psp_chess_new_game();
}

void
psp_main_menu_resign_game(void)
{
  psp_chess_resign_game();
}

static void
psp_main_menu_validate(void)
{
  int need_to_reset = 0;
  GCHESS.psp_cpu_clock = psp_cpu_clock;
 
  if (chess_player_color != GCHESS.chess_player_color) {
    psp_chess_change_color(chess_player_color);
  }
  if (chess_search_depth != GCHESS.chess_search_depth) {
    psp_chess_change_search_depth(chess_search_depth);
  }
  if (chess_book != GCHESS.chess_book) {
    psp_chess_change_book(chess_book);
  }
# if 0 //LUDO: TO_BE_DONE 
  if ((chess_move_limit != GCHESS.chess_move_limit) ||
      (chess_time_limit != GCHESS.chess_time_limit)) {
    psp_chess_change_limit(chess_time_limit, chess_move_limit);
  }
# endif
  GCHESS.chess_game_mode = chess_game_mode;
  GCHESS.chess_show_coord = chess_show_coord;
  GCHESS.chess_side_color = chess_side_color;

  if (need_to_reset) {
    psp_main_menu_new_game();
  }
  scePowerSetClockFrequency(GCHESS.psp_cpu_clock, GCHESS.psp_cpu_clock, GCHESS.psp_cpu_clock/2);
}


static void
psp_main_menu_save_config()
{
  int error;

  psp_main_menu_validate();
  error = psp_save_configuration();

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270, 60, "File saved !", PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back2_print(270, 60, "Can't save file !", PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
}

static void
psp_main_menu_screenshot(void)
{
  GCHESS.psp_screenshot_mode = 1;
}

int
psp_main_menu_exit(void)
{
  SceCtrlData c;

  psp_display_screen_menu();
  psp_sdl_back2_print(270,  60, "press X to confirm !", PSP_MENU_WARNING_COLOR);
  psp_menu_dirty = 1;
  psp_sdl_flip();

  psp_kbd_wait_no_button();

  do
  {
    sceCtrlReadBufferPositive(&c, 1);
    c.Buttons &= PSP_ALL_BUTTON_MASK;

    if (c.Buttons & PSP_CTRL_CROSS) psp_sdl_exit(0);

  } while (c.Buttons == 0);

  psp_kbd_wait_no_button();

  return 0;
}

static void
psp_main_menu_init(void)
{
  psp_cpu_clock       = GCHESS.psp_cpu_clock;
  chess_game_mode     = GCHESS.chess_game_mode;
  chess_search_depth  = GCHESS.chess_search_depth;
# if 0 //LUDO: TO_BE_DONE !
  chess_time_limit    = GCHESS.chess_time_limit;
  chess_move_limit    = GCHESS.chess_move_limit;
# endif
  chess_book          = GCHESS.chess_book;
  chess_player_color  = GCHESS.chess_player_color;
  chess_show_coord    = GCHESS.chess_show_coord;
}


void
psp_main_menu_player_color( int step )
{
  if (chess_player_color == black) chess_player_color = white;
  else                             chess_player_color = black;
}

void
psp_main_menu_side_color( int step )
{
  if (chess_side_color == black) chess_side_color = white;
  else                           chess_side_color = black;
}

void
psp_main_menu_search_depth( int step )
{
  if (step > 0) {
    if (chess_search_depth <= GCHESS_MAX_DEPTH) chess_search_depth++;
    else                                chess_search_depth = 0;
  } else 
  if (step < 0) {
    if (chess_search_depth > 0) chess_search_depth--;
    else                 chess_search_depth = GCHESS_MAX_DEPTH;
  }
}

# if 0 //LUDO: TO_BE_DONE !
void
psp_main_menu_time_limit( int step )
{
  if (step > 0) {
    if (chess_time_limit <= GCHESS_MAX_TIME) chess_time_limit++;
    else                                chess_time_limit = 0;
  } else 
  if (step < 0) {
    if (chess_time_limit > 0) chess_time_limit--;
    else                 chess_time_limit = GCHESS_MAX_TIME;
  }
}

void
psp_main_menu_move_limit( int step )
{
  if (step > 0) {
    if (chess_move_limit <= GCHESS_MAX_TIME) chess_move_limit++;
    else                                chess_move_limit = 0;
  } else 
  if (step < 0) {
    if (chess_move_limit > 0) chess_move_limit--;
    else                 chess_move_limit = GCHESS_MAX_TIME;
  }
}
# endif

void
psp_main_menu_book( int step )
{
  if (step > 0) {
    if (chess_book < BOOKPREFER) chess_book++;
    else                         chess_book = 0;
  } else 
  if (step < 0) {
    if (chess_book > 0) chess_book--;
    else                chess_book = BOOKPREFER;
  }
}

void
psp_main_menu_game_mode( int step )
{
# if 0 //LUDO:
  if (step > 0) {
    if (chess_game_mode < GCHESS_COMPUTER_VS_COMPUTER) chess_game_mode++;
    else                                               chess_game_mode = 0;
  } else {
    if (chess_game_mode > 0) chess_game_mode--;
    else                  chess_game_mode = GCHESS_COMPUTER_VS_COMPUTER;
  }
# else
  chess_game_mode = ! chess_game_mode;
# endif
}

void
psp_main_menu_show_coord( int step )
{
  chess_show_coord = ! chess_show_coord;
}

#define MAX_CLOCK_VALUES 5
static int clock_values[MAX_CLOCK_VALUES] = { 133, 222, 266, 300, 333 };

static void
psp_main_menu_clock( int step )
{
  int index;
  for (index = 0; index < MAX_CLOCK_VALUES; index++) {
    if (psp_cpu_clock == clock_values[index]) break;
  }
  if (step > 0) {
    index++;
    if (index >= MAX_CLOCK_VALUES) index = 0;
    psp_cpu_clock = clock_values[index];

  } else {
    index--;

    if (index < 0) index = MAX_CLOCK_VALUES - 1;
    psp_cpu_clock = clock_values[index];
  }
}


int 
psp_main_menu(void)
{
  SceCtrlData c;
  long        new_pad;
  long        old_pad;
  int         last_time;
  int         end_menu;

  psp_kbd_wait_no_button();

  old_pad   = 0;
  last_time = 0;
  end_menu  = 0;

  psp_main_menu_init();

  psp_menu_dirty = 1;

  while (! end_menu)
  {
    psp_display_screen_menu();

    psp_sdl_flip();

    while (1) {

      sceCtrlPeekBufferPositive(&c, 1);
      c.Buttons &= PSP_ALL_BUTTON_MASK;

      if (c.Buttons) break;
    }

    new_pad = c.Buttons;

    if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > PSP_MENU_MIN_TIME)) {
      last_time = c.TimeStamp;
      old_pad = new_pad;

    } else continue;

    if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) ==
        (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) {
      /* Exit ! */
      psp_sdl_exit(0);
    } else
    if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_SELECT)) == (PSP_CTRL_LTRIGGER|PSP_CTRL_SELECT)) {
      /* Ir shell ? */
      sceKernelDelayThread(100);
    } else
    if ((c.Buttons & PSP_CTRL_RTRIGGER) == PSP_CTRL_RTRIGGER) {
      psp_main_menu_new_game();
      end_menu = 1;
    } else
    if ((new_pad & PSP_CTRL_CROSS ) || 
        (new_pad & PSP_CTRL_CIRCLE) || 
        (new_pad & PSP_CTRL_RIGHT ) ||
        (new_pad & PSP_CTRL_LEFT  )) 
    {
      int step;

      if (new_pad & PSP_CTRL_LEFT)  step = -1;
      else 
      if (new_pad & PSP_CTRL_RIGHT) step =  1;
      else                          step =  0;

      switch (cur_menu_id ) 
      {
        case MENU_PLAYER_COLOR : psp_main_menu_player_color( step );
        break;
        case MENU_SIDE_COLOR   : psp_main_menu_side_color( step );
        break;
        case MENU_DEPTH        : psp_main_menu_search_depth( step );
        break;
        case MENU_BOOK         : psp_main_menu_book( step );
        break;
        case MENU_SHOW_COORD   : psp_main_menu_show_coord( step );
        break;
        case MENU_GAME_MODE    : psp_main_menu_game_mode( step );
        break;
# if 0 //LUDO:
        case MENU_TIME_LIMIT   : psp_main_menu_time_limit( step );
        break;
        case MENU_MOVE_LIMIT   : psp_main_menu_move_limit( step );
        break;
# endif
        case MENU_CLOCK        : psp_main_menu_clock( step );
        break;
        case MENU_SCREENSHOT   : psp_main_menu_screenshot();
                                 end_menu = 1;
        break;              
        case MENU_NEW_GAME     : psp_main_menu_new_game();
                                 end_menu = 1;
        break;                 
        case MENU_RESIGN_GAME  : psp_main_menu_resign_game();
                                 end_menu = 1;
        break;                 
        case MENU_SAVE_CFG     : psp_main_menu_save_config();
        break;               
        case MENU_BACK         : end_menu = 1;
        break;                 
        case MENU_EXIT         : psp_main_menu_exit();
        break;                 
        case MENU_HELP         : psp_menu_dirty = 1;
                                 psp_help_menu();
                                 old_pad = new_pad = 0;
                                 psp_menu_dirty = 1;
        break;              
      }

    } else
    if(new_pad & PSP_CTRL_UP) {

      if (cur_menu_id > 0) cur_menu_id--;
      else                 cur_menu_id = MAX_MENU_ITEM-1;

    } else
    if(new_pad & PSP_CTRL_DOWN) {

      if (cur_menu_id < (MAX_MENU_ITEM-1)) cur_menu_id++;
      else                                 cur_menu_id = 0;

    } else  
    if(new_pad & PSP_CTRL_SQUARE) {
      /* Cancel */
      end_menu = -1;
    } else 
    if((new_pad & PSP_CTRL_SELECT) == PSP_CTRL_SELECT) {
      /* Back to game */
      end_menu = 1;
    }
  }

  if (end_menu > 0) {
    psp_main_menu_validate();
  }
 
  psp_kbd_wait_no_button();

  psp_sdl_clear_screen( PSP_MENU_BLACK_COLOR );
  psp_sdl_flip();
  psp_sdl_clear_screen( PSP_MENU_BLACK_COLOR );
  psp_sdl_flip();

  return 1;
}

